@@ -1,4 +1,4 @@
-gaussian.CARlinear <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.nu2=NULL, prior.tau2=NULL, fix.rho.slo=FALSE, rho.slo=NULL, fix.rho.int=FALSE, rho.int=NULL,  verbose=TRUE)
+gaussian.CARlinear <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.nu2=NULL, prior.tau2=NULL, rho.slo=NULL, rho.int=NULL,  verbose=TRUE)
 {
 ##############################################
 #### Format the arguments and check for errors
@@ -18,43 +18,41 @@ X.mean <- frame.results$X.mean
 X.indicator <- frame.results$X.indicator 
 offset <- frame.results$offset
 Y <- frame.results$Y
-Y.miss <- frame.results$Y.miss
 which.miss <- frame.results$which.miss
 n.miss <- frame.results$n.miss  
-X.short <- X.standardised[which.miss==1, ]  
+Y.DA <- Y  
     
     
 #### Check on the rho arguments
-    if(!is.logical(fix.rho.int)) stop("fix.rho.int is not logical.", call.=FALSE)   
-    if(fix.rho.int & is.null(rho.int)) stop("rho.int is fixed but an initial value was not set.", call.=FALSE)   
-    if(fix.rho.int & !is.numeric(rho.int) ) stop("rho.S is not numeric.", call.=FALSE)  
-    if(!is.logical(fix.rho.slo)) stop("fix.rho.slo is not logical.", call.=FALSE)   
-    if(fix.rho.slo & is.null(rho.slo)) stop("rho.slo is fixed but an initial value was not set.", call.=FALSE)   
-    if(fix.rho.slo & !is.numeric(rho.slo) ) stop("rho.slo is not numeric.", call.=FALSE)  
-    if(fix.rho.int)
+    if(is.null(rho.int))
+    {
+    rho <- runif(1)
+    fix.rho.int <- FALSE   
+    }else
     {
     rho <- rho.int
-    }else
-    {
-    rho <- runif(1)       
+    fix.rho.int <- TRUE
     }
-    if(fix.rho.slo)
+    if(!is.numeric(rho)) stop("rho.int is fixed but is not numeric.", call.=FALSE)  
+    if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)    
+
+    if(is.null(rho.slo))
+    {
+    lambda <- runif(1)
+    fix.rho.slo <- FALSE   
+    }else
     {
     lambda <- rho.slo
-    }else
-    {
-    lambda <- runif(1)       
-    }   
-    if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
-    if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    fix.rho.slo <- TRUE
+    }
+    if(!is.numeric(lambda)) stop("rho.slo is fixed but is not numeric.", call.=FALSE)  
     if(lambda<0 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
     if(lambda>1 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
 
-    
+
 #### CAR quantities
-rho.sum <- rho + lambda
-fix.rho.sum <- as.logical(fix.rho.int * fix.rho.slo)
-W.quants <- common.Wcheckformat.leroux(W, fix.rho.sum, rho.sum)
+W.quants <- common.Wcheckformat.leroux(W)
 K <- W.quants$n
 N <- N.all / K
 W <- W.quants$W
@@ -67,11 +65,11 @@ W.begfin <- W.quants$W.begfin
     
 #### Priors
     if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+    if(is.null(prior.var.beta)) prior.var.beta <- rep(100000, p)
     if(is.null(prior.tau2)) prior.tau2 <- c(1, 0.01)
     if(is.null(prior.nu2)) prior.nu2 <- c(1, 0.01)
     if(is.null(prior.mean.alpha)) prior.mean.alpha <- rep(0, 1)
-    if(is.null(prior.var.alpha)) prior.var.alpha <- rep(1000, 1)
+    if(is.null(prior.var.alpha)) prior.var.alpha <- rep(100000, 1)
 prior.beta.check(prior.mean.beta, prior.var.beta, p)
 prior.var.check(prior.tau2)
 prior.var.check(prior.nu2)
@@ -107,8 +105,19 @@ phi <- rnorm(n=K, mean=0, sd = res.sd)
 delta <- rnorm(n=K, mean=0, sd = res.sd)
 tau2.phi <- var(phi)/10
 tau2.delta <- var(delta)/10
+nu2 <- runif(1, 0, res.sd)
+
+
+#### Specify matrix quantities
+offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
+regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)   
+phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)
+time.mat <- matrix(rep(time, K), byrow=TRUE, nrow=K)    
+delta.time.mat <- apply(time.mat, 2, "*", delta)
+alpha.offset1 <- sum(time.mat^2)
+fitted <- as.numeric(offset.mat + regression.mat + phi.mat + delta.time.mat + alpha * time.mat)
+
 
- 
 
 ###############################    
 #### Set up the MCMC quantities    
@@ -125,8 +134,7 @@ samples.nu2 <- array(NA, c(n.keep, 1))
 samples.tau2 <- array(NA, c(n.keep, 2))
 colnames(samples.tau2) <- c("tau2.int", "tau2.slo")
 samples.fitted <- array(NA, c(n.keep, N.all))
-samples.like <- array(NA, c(n.keep, N.all))
-samples.deviance <- array(NA, c(n.keep, 1))
+samples.loglike <- array(NA, c(n.keep, N.all))
     if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
 
     
@@ -156,23 +164,6 @@ tau2.delta.shape <- prior.tau2[1] + K/2
     if(!fix.rho.slo) det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))     
 
 
-    
-##########################################
-#### Specify quantities that do not change
-########################################## 
-offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
-regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)   
-Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE)
-Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=N, byrow=FALSE)
-which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
-phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)
-time.mat <- matrix(rep(time, K), byrow=TRUE, nrow=K)    
-delta.time.mat <- apply(time.mat, 2, "*", delta)
-alpha.offset1 <- sum(time.mat^2 * which.miss.mat)
-ntime.miss <- apply(which.miss.mat,1,sum)
-ntime.miss2 <- apply(which.miss.mat * time.mat^2,1,sum)
-
-
 #### Check for islands
 W.list<- mat2listw(W)
 W.nb <- W.list$neighbours
@@ -184,7 +175,7 @@ n.islands <- max(W.islands$nc)
 
 
 #### Beta update quantities
-data.precision.beta <- t(X.short) %*% X.short
+data.precision.beta <- t(X.standardised) %*% X.standardised
     if(length(prior.var.beta)==1)
     {
     prior.precision.beta <- 1 / prior.var.beta
@@ -201,7 +192,7 @@ data.precision.beta <- t(X.short) %*% X.short
 #### Start timer
     if(verbose)
     {
-    cat("Generating", n.keep, "post burnin and thinned (if requested) samples\n", sep = " ")
+    cat("Generating", n.keep, "post burnin and thinned (if requested) samples.\n", sep = " ")
     progressBar <- txtProgressBar(style = 3)
     percentage.points<-round((1:100/100)*n.sample)
     }else
@@ -213,10 +204,22 @@ data.precision.beta <- t(X.short) %*% X.short
 #### Create the MCMC samples    
     for(j in 1:n.sample)
     {
+    ####################################
+    ## Sample from Y - data augmentation
+    ####################################
+        if(n.miss>0)
+        {
+        Y.DA[which.miss==0] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))    
+        }else
+        {}
+    Y.DA.mat <- matrix(Y.DA, nrow=K, ncol=N, byrow=FALSE)
+        
+        
+        
     ##################
     ## Sample from nu2
     ##################
-    nu2.offset <- as.numeric(Y.mat - offset.mat - regression.mat - phi.mat - delta.time.mat - alpha * time.mat)[which.miss==1]
+    nu2.offset <- as.numeric(Y.DA.mat - offset.mat - regression.mat - phi.mat - delta.time.mat - alpha * time.mat)
     nu2.scale <- prior.nu2[2]  + sum(nu2.offset^2)/2
     nu2 <- 1 / rgamma(1, nu2.shape, scale=(1/nu2.scale)) 
 
@@ -227,8 +230,8 @@ data.precision.beta <- t(X.short) %*% X.short
     ####################
     fc.precision <- prior.precision.beta + data.precision.beta / nu2
     fc.var <- solve(fc.precision)
-    beta.offset <- as.numeric(Y.mat - offset.mat - phi.mat - delta.time.mat - alpha * time.mat)[which.miss==1]
-    beta.offset2 <- t(X.short) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
+    beta.offset <- as.numeric(Y.DA.mat - offset.mat - phi.mat - delta.time.mat - alpha * time.mat)
+    beta.offset2 <- t(X.standardised) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
     fc.mean <- fc.var %*% beta.offset2
     chol.var <- t(chol(fc.var))
     beta <- fc.mean + chol.var %*% rnorm(p)        
@@ -240,7 +243,7 @@ data.precision.beta <- t(X.short) %*% X.short
     ## Sample from alpha
     ####################
     fc.var <- 1 / (1 / prior.var.alpha + alpha.offset1 / nu2)
-    alpha.offset <- (Y.mat - offset.mat - regression.mat - phi.mat - delta.time.mat) * time.mat
+    alpha.offset <- (Y.DA.mat - offset.mat - regression.mat - phi.mat - delta.time.mat) * time.mat
     alpha.offset2 <- sum(alpha.offset, na.rm=TRUE) / nu2
     fc.mean <- fc.var * (alpha.offset2 +  prior.mean.alpha / prior.var.alpha)
     alpha <- rnorm(n=1, mean=fc.mean, sd=sqrt(fc.var))
@@ -250,9 +253,9 @@ data.precision.beta <- t(X.short) %*% X.short
     ####################
     ## Sample from phi
     ####################
-    phi.offset <- Y.mat - offset.mat - regression.mat - delta.time.mat - alpha * time.mat
+    phi.offset <- Y.DA.mat - offset.mat - regression.mat - delta.time.mat - alpha * time.mat
     phi.offset2 <- apply(phi.offset,1, sum, na.rm=TRUE)
-    temp1 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, nu2, phi.offset2, rho, ntime.miss)
+    temp1 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, nu2, phi.offset2, rho, N)
     phi <- temp1
         if(rho<1)
         {
@@ -268,9 +271,9 @@ data.precision.beta <- t(X.short) %*% X.short
     ####################
     ## Sample from delta
     ####################
-    delta.offset <- (Y.mat - offset.mat - regression.mat - phi.mat - alpha * time.mat) * time.mat
+    delta.offset <- (Y.DA.mat - offset.mat - regression.mat - phi.mat - alpha * time.mat) * time.mat
     delta.offset2 <- apply(delta.offset,1, sum, na.rm=TRUE)
-    temp2 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta, nu2, delta.offset2, lambda, ntime.miss2)
+    temp2 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta, nu2, delta.offset2, lambda, sum(time^2))
     delta <- temp2
     if(lambda <1)
     {
@@ -356,9 +359,7 @@ data.precision.beta <- t(X.short) %*% X.short
     ## Calculate the deviance
     #########################
     fitted <- as.numeric(offset.mat + regression.mat + phi.mat + delta.time.mat + alpha * time.mat)
-    deviance.all <- dnorm(Y, mean = fitted, sd = rep(sqrt(nu2),N.all), log=TRUE)
-    like <- exp(deviance.all)
-    deviance <- -2 * sum(deviance.all, na.rm=TRUE)      
+    loglike <- dnorm(Y, mean = fitted, sd = rep(sqrt(nu2),N.all), log=TRUE)
         
     
     
@@ -376,10 +377,9 @@ data.precision.beta <- t(X.short) %*% X.short
             if(!fix.rho.slo) samples.lambda[ele, ] <- lambda
         samples.nu2[ele, ] <- nu2
         samples.tau2[ele, ] <- c(tau2.phi, tau2.delta)
-        samples.deviance[ele, ] <- deviance
         samples.fitted[ele, ] <- fitted
-        samples.like[ele, ] <- like
-            if(n.miss>0) samples.Y[ele, ] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))
+        samples.loglike[ele, ] <- loglike
+            if(n.miss>0) samples.Y[ele, ] <- Y.DA[which.miss==0]
         }else
         {}
         
@@ -413,7 +413,7 @@ data.precision.beta <- t(X.short) %*% X.short
 #### end timer
     if(verbose)
     {
-    cat("\nSummarising results")
+    cat("\nSummarising results.")
     close(progressBar)
     }else
     {}
@@ -442,43 +442,29 @@ accept.final <- c(rep(100,4), accept.rho, accept.lambda)
 names(accept.final) <- c("beta", "alpha", "phi", "delta", "rho.int", "rho.slo")
     
       
-#### Deviance information criterion (DIC)
-median.phi <- apply(samples.phi, 2, median)
-median.delta <- apply(samples.delta, 2, median)
-median.alpha <- median(samples.alpha)
-median.phi.mat <- matrix(rep(median.phi, N), byrow=F, nrow=K)
-delta.time.mat <- apply(time.mat, 2, "*", median.delta)
-median.beta <- apply(samples.beta,2,median)
-regression.mat <- matrix(X.standardised %*% median.beta, nrow=K, ncol=N, byrow=FALSE)   
-LP <- offset.mat + regression.mat + phi.mat + delta.time.mat + median.alpha * time.mat
-fitted.median <- as.numeric(LP)
-nu2.median <- median(samples.nu2)
-deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),N.all), log = TRUE), na.rm=TRUE)
-p.d <- median(samples.deviance) - deviance.fitted
-DIC <- 2 * median(samples.deviance) - deviance.fitted    
+#### Compute the fitted deviance
+mean.phi <- apply(samples.phi, 2, mean)
+mean.delta <- apply(samples.delta, 2, mean)
+mean.alpha <- mean(samples.alpha)
+mean.phi.mat <- matrix(rep(mean.phi, N), byrow=F, nrow=K)
+delta.time.mat <- apply(time.mat, 2, "*", mean.delta)
+mean.beta <- apply(samples.beta,2,mean)
+regression.mat <- matrix(X.standardised %*% mean.beta, nrow=K, ncol=N, byrow=FALSE)   
+LP <- offset.mat + regression.mat + mean.phi.mat + delta.time.mat + mean.alpha * time.mat
+fitted.mean <- as.numeric(LP)
+nu2.mean <- mean(samples.nu2)
+deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.mean, sd = rep(sqrt(nu2.mean),N.all), log = TRUE), na.rm=TRUE)
 
 
-#### Watanabe-Akaike Information Criterion (WAIC)
-LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-WAIC <- -2 * (LPPD - p.w)
+#### Model fit criteria
+modelfit <- common.modelfit(samples.loglike, deviance.fitted)
 
 
-#### Compute the Conditional Predictive Ordinate  
-CPO <- rep(NA, N.all)
-    for(j in 1:N.all)
-    {
-    CPO[j] <- 1/median((1 / dnorm(Y[j], mean=samples.fitted[ ,j], sd=sqrt(samples.nu2))))    
-    }
-LMPL <- sum(log(CPO), na.rm=TRUE)  
-
-  
-## Create the fitted values and residuals
-fitted.values <- apply(samples.fitted, 2, median)
+#### Create the fitted values and residuals
+fitted.values <- apply(samples.fitted, 2, mean)
 response.residuals <- as.numeric(Y) - fitted.values
-pearson.residuals <- response.residuals /sqrt(nu2.median)
-deviance.residuals <- sign(response.residuals) * sqrt((Y-fitted.values)^2/nu2.median)
-residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
+pearson.residuals <- response.residuals /sqrt(nu2.mean)
+residuals <- data.frame(response=response.residuals, pearson=pearson.residuals)
     
     
 #### transform the parameters back to the origianl covariate scale.
@@ -527,11 +513,6 @@ summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
     
     
 #### Compile and return the results
-loglike <- (-0.5 * deviance.fitted)
-modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike)
-names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood")
-
-
 #### Harmonise samples in case of them not being generated
     if(fix.rho.int & fix.rho.slo)
     {
@@ -561,7 +542,7 @@ class(results) <- "CARBayesST"
     if(verbose)
     {
     b<-proc.time()
-    cat(" finished in ", round(b[3]-a[3], 1), "seconds")
+    cat("Finished in ", round(b[3]-a[3], 1), "seconds.\n")
     }else
     {}
 return(results)
